@@ -1,7 +1,7 @@
 // SPDX-License-Identifier: (GPL-2.0-or-later OR BSD-2-Clause)
 /*
  * Traceshark - a visualizer for visualizing ftrace and perf traces
- * Copyright (C) 2015-2020  Viktor Rosendahl <viktor.rosendahl@gmail.com>
+ * Copyright (C) 2015-2022  Viktor Rosendahl <viktor.rosendahl@gmail.com>
  *
  * This file is dual licensed: you can use it either under the terms of
  * the GPL, or the BSD license, at your option.
@@ -86,6 +86,10 @@ public:
 	QVector<int>    schedEventIdx;
 	vtl::BitVector  schedData;
 	QVector<double> scaledSchedData;
+	QVector<double> delayTimev;
+	QVector<double> delay;
+	QVector<double> delayHeight;
+	QVector<double> delayZero;
 	QVector<double> wakeTimev;
 	QVector<double> wakeDelay;
 	QVector<double> wakeHeight;
@@ -112,7 +116,7 @@ public:
 	bool doScale();
 	bool doStats();
 	bool doStatsTimeLimited();
-	bool doScaleWakeup();
+	bool doScaleDelay();
 	bool doScaleRunning();
 	bool doScalePreempted();
 	bool doScaleUnint();
@@ -129,7 +133,7 @@ private:
 	vtl_always_inline int binarySearch(const vtl::Time &time);
 	int findLower(const vtl::Time &time);
 	int findHigher(const vtl::Time &time);
-	bool fillDataVector(QVector<double> &timev, QVector<double> &data,
+	void fillDataVector(QVector<double> &timev, QVector<double> &data,
 			    QVector<double> *zerov, double height);
 protected:
 	static vtl::Time lowerTimeLimit;
